/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *
 * The Javadoc comments are taken from Crimson 1.1 developed by The Apache Software Foundation
 * and are made available under the terms and conditions of the Apache Software License 1.1
 * which accompanies this distribution and is available at
 * http://www.apache.org/licenses/LICENSE-1.1.
 *
 * All other portions of this file including the functional Java code, are made available under 
 * the terms and conditions of the EPL.
 *******************************************************************************/ 
package javax.xml.parsers;

/**
 * Defines a factory API that enables applications to obtain a
 * parser that produces DOM object trees from XML documents.
 *
 * An implementation of the <code>DocumentBuilderFactory</code> class is
 * <em>NOT</em> guaranteed to be thread safe. It is up to the user application 
 * to make sure about the use of the <code>DocumentBuilderFactory</code> from 
 * more than one thread. Alternatively the application can have one instance 
 * of the <code>DocumentBuilderFactory</code> per thread.
 * An application can use the same instance of the factory to obtain one or 
 * more instances of the <code>DocumentBuilder</code> provided the instance
 * of the factory isn't being used in more than one thread at a time.
 *
 * @since JAXP 1.0
 * @version 1.0
 */
public abstract class DocumentBuilderFactory {
	boolean namespaceAware;
	boolean validating;
	boolean coalescing;
	boolean ignoreComments;
	boolean expandEntityRef;
	boolean whitespace;
/**
 * Obtain a new instance of a
 * <code>DocumentBuilderFactory</code>. This static method creates
 * a new factory instance.
 * This method uses the following ordered lookup procedure to determine
 * the <code>DocumentBuilderFactory</code> implementation class to
 * load:
 * <ul>
 * <li>
 * Use the <code>javax.xml.parsers.DocumentBuilderFactory</code> system
 * property.
 * </li>
 * <li>
 * Use the properties file "lib/jaxp.properties" in the JRE directory.
 * This configuration file is in standard <code>java.util.Properties
 * </code> format and contains the fully qualified name of the
 * implementation class with the key being the system property defined
 * above.
 * </li>
 * <li>
 * Use the Services API (as detailed in the JAR specification), if
 * available, to determine the classname. The Services API will look
 * for a classname in the file
 * <code>META-INF/services/javax.xml.parsers.DocumentBuilderFactory</code>
 * in jars available to the runtime.
 * </li>
 * <li>
 * Platform default <code>DocumentBuilderFactory</code> instance.
 * </li>
 * </ul>
 *
 * Once an application has obtained a reference to a
 * <code>DocumentBuilderFactory</code> it can use the factory to
 * configure and obtain parser instances.
 *
 * @exception FactoryConfigurationError if the implementation is not
 * available or cannot be instantiated.
 */

public static DocumentBuilderFactory newInstance() throws FactoryConfigurationError {
	return (DocumentBuilderFactory) FactoryFinder.find(
				/* The default property name according to the JAXP spec */
				"javax.xml.parsers.DocumentBuilderFactory", //$NON-NLS-1$
				/* The fallback implementation class name */
				"org.eclipse.ercp.jaxp.implForCore.DocumentBuilderFactoryImpl"); //$NON-NLS-1$
}
protected DocumentBuilderFactory() {
	super();
	this.namespaceAware = false;
	this.validating = false;
	this.coalescing = false;
	this.ignoreComments = true;
	this.expandEntityRef = false;
	this.whitespace = false;
}
/**
 * Allows the user to retrieve specific attributes on the underlying
 * implementation.
 * @param name The name of the attribute.
 * @return value The value of the attribute.
 * @exception IllegalArgumentException thrown if the underlying
 * implementation doesn't recognize the attribute.
 */
public abstract Object getAttribute(String name) throws IllegalArgumentException;
/**
 * Allows the user to set specific attributes on the underlying
 * implementation.
 * @param name The name of the attribute.
 * @param value The value of the attribute.
 * @exception IllegalArgumentException thrown if the underlying
 * implementation doesn't recognize the attribute.
 */
public abstract void setAttribute(String name, Object value) throws IllegalArgumentException;
/**
 * Indicates whether or not the factory is configured to produce
 * parsers which converts CDATA nodes to Text nodes and appends it to
 * the adjacent (if any) Text node.
 *
 * @return  true if the factory is configured to produce parsers
 *          which converts CDATA nodes to Text nodes and appends it to
 *          the adjacent (if any) Text node; false otherwise.
 */
public boolean isCoalescing() {
    return coalescing;
}
/**
 * Specifies that the parser produced by this code will
 * convert CDATA nodes to Text nodes and append it to the
 * adjacent (if any) text node. By default the value of this is set to
 * <code>false</code>
 *
 * @param coalescing  true if the parser produced will convert CDATA nodes
 *                    to Text nodes and append it to the adjacent (if any)
 *                    text node; false otherwise.
 */
public void setCoalescing(boolean coalescing) {
    this.coalescing = coalescing;
}
/**
 * Indicates whether or not the factory is configured to produce
 * parsers which expand entity reference nodes.
 *
 * @return  true if the factory is configured to produce parsers
 *          which expand entity reference nodes; false otherwise.
 */
public boolean isExpandEntityReferences() {
    return expandEntityRef;
}
/**
 * Specifies that the parser produced by this code will
 * expand entity reference nodes. By default the value of this is set to
 * <code>true</code>
 *
 * @param expandEntityRef true if the parser produced will expand entity
 *                        reference nodes; false otherwise.
 */
public void setExpandEntityReferences(boolean expandEntityRef) {
    this.expandEntityRef = expandEntityRef;
}
/**
 * Indicates whether or not the factory is configured to produce
 * parsers which ignores comments.
 *
 * @return  true if the factory is configured to produce parsers
 *          which ignores comments; false otherwise.
 */

public boolean isIgnoringComments() {
    return ignoreComments;
}
/**
 * Specifies that the parser produced by this code will
 * ignore comments. By default the value of this is set to <code>false
 * </code>
 */

public void setIgnoringComments(boolean ignoreComments) {
    this.ignoreComments = ignoreComments;
}
/**
 * Indicates whether or not the factory is configured to produce
 * parsers which ignore ignorable whitespace in element content.
 *
 * @return  true if the factory is configured to produce parsers
 *          which ignore ignorable whitespace in element content;
 *          false otherwise.
 */
public boolean isIgnoringElementContentWhitespace() {
    return whitespace;
}
/**
 * Specifies that the parsers created by this  factory must eliminate
 * whitespace in element content (sometimes known loosely as
 * 'ignorable whitespace') when parsing XML documents (see XML Rec
 * 2.10). Note that only whitespace which is directly contained within
 * element content that has an element only content model (see XML
 * Rec 3.2.1) will be eliminated. Due to reliance on the content model
 * this setting requires the parser to be in validating mode. By default
 * the value of this is set to <code>false</code>.
 *
 * @param whitespace true if the parser created must eliminate whitespace
 *                   in the element content when parsing XML documents;
 *                   false otherwise.
 */

public void setIgnoringElementContentWhitespace(boolean whitespace) {
    this.whitespace = whitespace;
}
/**
 * Indicates whether or not the factory is configured to produce
 * parsers which are namespace aware.
 *
 * @return  true if the factory is configured to produce parsers which
 *          are namespace aware; false otherwise.
 */

public boolean isNamespaceAware() {
    return namespaceAware;
}
/**
 * Specifies that the parser produced by this code will
 * provide support for XML namespaces. By default the value of this is set
 * to <code>false</code>
 *
 * @param awareness true if the parser produced will provide support
 *                  for XML namespaces; false otherwise.
 */

public void setNamespaceAware(boolean awareness) {
    this.namespaceAware = awareness;
}
/**
 * Indicates whether or not the factory is configured to produce
 * parsers which validate the XML content during parse.
 *
 * @return  true if the factory is configured to produce parsers
 *          which validate the XML content during parse; false otherwise.
 */
public boolean isValidating() {
    return validating;
}
/**
 * Specifies that the parser produced by this code will
 * validate documents as they are parsed. By default the value of this
 * is set to <code>false</code>.
 *
 * @param validating true if the parser produced will validate documents
 *                   as they are parsed; false otherwise.
 */

public void setValidating(boolean validating) {
    this.validating = validating;
}
/**
 * Creates a new instance of a {@link javax.xml.parsers.DocumentBuilder}
 * using the currently configured parameters.
 *
 * @exception ParserConfigurationException if a DocumentBuilder
 * cannot be created which satisfies the configuration requested.
 * @return A new instance of a DocumentBuilder.
 */
public abstract DocumentBuilder newDocumentBuilder() throws ParserConfigurationException;
}
